# -*- coding: utf-8 -*-
# MegEngine is Licensed under the Apache License, Version 2.0 (the "License")
#
# Copyright (c) 2014-2020 Megvii Inc. All rights reserved.
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT ARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
"""This python module contains functions to apply the operators defined by
megbrain.

.. note::
    Most of the functions are automatically generated, and their signature have
    the form contain a ``param`` argument (or more than one arguments such as
    :func:`convolution` that has ``param`` and ``execution_polity``) and also
    accept keyword arguments. In such case, it can be called by either
    providing a param object of appropriate type, or by passing the arguments
    needed by the constructor of param object to the keyword arguments.
    Furthermore, for a param that needs an enumeration member, the enum name
    can be used to refer to the enum object.

    For example, the following statements are equivalent::

        elemwise([a, b], mode='max')
        elemwise([a, b], mode=opr_param_defs.Elemwise.Mode.MAX)
        elemwise([a, b], param=opr_param_defs.Elemwise('max'))
"""

__git_commit__ = "a594f96b16a2126215d62695bbb2f92961f67c1a"

import collections

from . import helper
from .helper import PodOpVisitor
from . import param_defs
from ..._imperative_rt import OperatorNodeConfig as Config

__all__ = [
    "Elemwise",
    "Reduce",
    "TypeCvt",
    "MatrixMul",
    "BatchedMatrixMul",
    "Dot",
    "MatrixInverse",
    "SVD",
    "CondExecPredLogical",
    "CondExecMarkIfNeed",
    "Convolution",
    "MaskConvolution",
    "MaskPropagate",
    "Images2Neibs",
    "Local",
    "GroupLocal",
    "LRN",
    "Pooling",
    "AdaptivePooling",
    "ROIPooling",
    "Convolution3D",
    "ConvBiasForward",
    "BatchNorm",
    "LocalShareForward",
    "ROIAlign",
    "DeformableConvForward",
    "DeformablePSROIPoolingForward",
    "BatchConvBiasForward",
    "WarpPerspective",
    "Rotate",
    "CvtColor",
    "GaussianBlur",
    "Resize",
    "WarpAffine",
    "Remap",
    "IndexingOneHot",
    "IndexingSetOneHot",
    "IndexingRemap",
    "Copy",
    "Argmax",
    "Argmin",
    "Argsort",
    "CondTake",
    "TopK",
    "AffineInt",
    "ElemwiseMultiType",
    "UniformRNG",
    "GaussianRNG",
    "Alloc",
    "Linspace",
    "Eye",
    "GetVarShape",
    "Concat",
    "Broadcast",
    "RelayoutFormat",
    "MarkNoBroadcastElemwise",
    "Identity",
    "AssertEqual",
    "PersistentOutputStorage",
    "RequireInputDynamicStorage",
]


# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Elemwise(PodOpVisitor):
    param_names = ("param",)
    name = "Elemwise"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Elemwise, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Reduce(PodOpVisitor):
    param_names = ("param",)
    name = "ReduceV2"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Reduce, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class TypeCvt(PodOpVisitor):
    param_names = ("param",)
    name = "TypeCvtV2"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.SerializedDType, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class MatrixMul(PodOpVisitor):
    param_names = ("param",)
    name = "MatrixMulV2"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.MatrixMul, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class BatchedMatrixMul(PodOpVisitor):
    param_names = ("param",)
    name = "BatchedMatrixMulV2"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.MatrixMul, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Dot(PodOpVisitor):
    param_names = ("param",)
    name = "Dot"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Empty, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class MatrixInverse(PodOpVisitor):
    param_names = ("param",)
    name = "MatrixInverse"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Empty, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class SVD(PodOpVisitor):
    param_names = ("param",)
    name = "SVD"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.SVD, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class CondExecPredLogical(PodOpVisitor):
    param_names = ("param",)
    name = "CondExecPredLogical"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.CondExecPredLogical, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class CondExecMarkIfNeed(PodOpVisitor):
    param_names = ("param",)
    name = "CondExecMarkIfNeed"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.CondExecMark, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Convolution(PodOpVisitor):
    param_names = ("param", "execution_polity",)
    name = "ConvolutionV1"

    def __init__(self, *, param=None, execution_polity=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.Convolution, kwargs)
        self.execution_polity = helper.make_param(execution_polity, param_defs.ExecutionPolicy, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class MaskConvolution(PodOpVisitor):
    param_names = ("param",)
    name = "MaskConvolutionV1"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Convolution, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class MaskPropagate(PodOpVisitor):
    param_names = ("param",)
    name = "MaskPropagate"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.MaskPropagate, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Images2Neibs(PodOpVisitor):
    param_names = ("param",)
    name = "Images2Neibs"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Images2Neibs, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Local(PodOpVisitor):
    param_names = ("param",)
    name = "LocalV1"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Convolution, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class GroupLocal(PodOpVisitor):
    param_names = ("param",)
    name = "GroupLocalV1"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Convolution, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class LRN(PodOpVisitor):
    param_names = ("param",)
    name = "LRN"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.LRN, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Pooling(PodOpVisitor):
    param_names = ("param",)
    name = "Pooling"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Pooling, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class AdaptivePooling(PodOpVisitor):
    param_names = ("param",)
    name = "AdaptivePooling"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.AdaptivePooling, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class ROIPooling(PodOpVisitor):
    param_names = ("param",)
    name = "ROIPooling"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.ROIPooling, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Convolution3D(PodOpVisitor):
    param_names = ("param", "execution_polity",)
    name = "Convolution3D"

    def __init__(self, *, param=None, execution_polity=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Convolution3D, kwargs)
        self.execution_polity = helper.make_param(execution_polity, param_defs.ExecutionPolicy, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class ConvBiasForward(PodOpVisitor):
    param_names = ("param", "execution_policy",)
    name = "ConvBiasForwardV3"

    def __init__(self, *, param=None, execution_policy=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.ConvBias, kwargs)
        self.execution_policy = helper.make_param(execution_policy, param_defs.ExecutionPolicy, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class BatchNorm(PodOpVisitor):
    param_names = ("param",)
    name = "BatchNorm"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.BN, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class LocalShareForward(PodOpVisitor):
    param_names = ("param", "execution_policy",)
    name = "LocalShareForward"

    def __init__(self, *, param=None, execution_policy=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.LocalShare, kwargs)
        self.execution_policy = helper.make_param(execution_policy, param_defs.ExecutionPolicy, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class ROIAlign(PodOpVisitor):
    param_names = ("param",)
    name = "ROIAlign"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.ROIAlign, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class DeformableConvForward(PodOpVisitor):
    param_names = ("param", "execution_policy",)
    name = "DeformableConvForward"

    def __init__(self, *, param=None, execution_policy=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.Convolution, kwargs)
        self.execution_policy = helper.make_param(execution_policy, param_defs.ExecutionPolicy, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class DeformablePSROIPoolingForward(PodOpVisitor):
    param_names = ("param",)
    name = "DeformablePSROIPoolingForward"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.DeformablePSROIPooling, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class BatchConvBiasForward(PodOpVisitor):
    param_names = ("param", "execution_policy",)
    name = "BatchConvBiasForward"

    def __init__(self, *, param=None, execution_policy=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.BatchConvBias, kwargs)
        self.execution_policy = helper.make_param(execution_policy, param_defs.ExecutionPolicy, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class WarpPerspective(PodOpVisitor):
    param_names = ("param",)
    name = "WarpPerspective"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.WarpPerspective, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Rotate(PodOpVisitor):
    param_names = ("param",)
    name = "Rotate"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Rotate, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class CvtColor(PodOpVisitor):
    param_names = ("param",)
    name = "CvtColor"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.CvtColor, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class GaussianBlur(PodOpVisitor):
    param_names = ("param",)
    name = "GaussianBlur"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.GaussianBlur, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Resize(PodOpVisitor):
    param_names = ("param",)
    name = "ResizeV1"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Resize, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class WarpAffine(PodOpVisitor):
    param_names = ("param",)
    name = "WarpAffineV1"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.WarpAffine, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Remap(PodOpVisitor):
    param_names = ("param",)
    name = "Remap"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Remap, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class IndexingOneHot(PodOpVisitor):
    param_names = ("axis",)
    name = "IndexingOneHot"

    def __init__(self, *, axis=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.axis = helper.make_param(axis, param_defs.Axis, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class IndexingSetOneHot(PodOpVisitor):
    param_names = ("axis",)
    name = "IndexingSetOneHot"

    def __init__(self, *, axis=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.axis = helper.make_param(axis, param_defs.Axis, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class IndexingRemap(PodOpVisitor):
    param_names = ("param",)
    name = "IndexingRemap"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.IndexingRemap, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Copy(PodOpVisitor):
    param_names = ("param",)
    name = "Copy"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Empty, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Argmax(PodOpVisitor):
    param_names = ("axis",)
    name = "Argmax"

    def __init__(self, *, axis=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.axis = helper.make_param(axis, param_defs.Axis, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Argmin(PodOpVisitor):
    param_names = ("axis",)
    name = "Argmin"

    def __init__(self, *, axis=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.axis = helper.make_param(axis, param_defs.Axis, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Argsort(PodOpVisitor):
    param_names = ("param",)
    name = "Argsort"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Argsort, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class CondTake(PodOpVisitor):
    param_names = ("param",)
    name = "CondTake"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.CondTake, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class TopK(PodOpVisitor):
    param_names = ("param",)
    name = "TopK"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.TopK, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class AffineInt(PodOpVisitor):
    param_names = ("param",)
    name = "AffineInt"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Empty, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class ElemwiseMultiType(PodOpVisitor):
    param_names = ("param",)
    name = "ElemwiseMultiType"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, dtype=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        if dtype:
            config.dtype = dtype
        self.config = config

        self.param = helper.make_param(param, param_defs.ElemwiseMultiType, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class UniformRNG(PodOpVisitor):
    param_names = ("param",)
    name = "UniformRNG"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.UniformRNG, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class GaussianRNG(PodOpVisitor):
    param_names = ("param",)
    name = "GaussianRNG"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.GaussianRNG, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Alloc(PodOpVisitor):
    param_names = ("param",)
    name = "Alloc"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.DType, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Linspace(PodOpVisitor):
    param_names = ("param",)
    name = "Linspace"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Linspace, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Eye(PodOpVisitor):
    param_names = ("param",)
    name = "Eye"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Eye, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class GetVarShape(PodOpVisitor):
    param_names = ("axis",)
    name = "GetVarShapeV1"

    def __init__(self, *, axis=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.axis = helper.make_param(axis, param_defs.OptionalAxisV1, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Concat(PodOpVisitor):
    param_names = ("axis",)
    name = "Concat"

    def __init__(self, *, axis=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.axis = helper.make_param(axis, param_defs.Axis, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Broadcast(PodOpVisitor):
    param_names = ("param",)
    name = "Broadcast"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Empty, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class RelayoutFormat(PodOpVisitor):
    param_names = ("param",)
    name = "RelayoutFormat"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.RelayoutFormat, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class MarkNoBroadcastElemwise(PodOpVisitor):
    param_names = ("param",)
    name = "MarkNoBroadcastElemwise"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Empty, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class Identity(PodOpVisitor):
    param_names = ("param",)
    name = "Identity"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Empty, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class AssertEqual(PodOpVisitor):
    param_names = ("param",)
    name = "AssertEqual"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.AssertEqual, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class PersistentOutputStorage(PodOpVisitor):
    param_names = ("param",)
    name = "PersistentOutputStorage"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.PersistentOutputStorage, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)

# /Users/mac_ci/megengine_github_mirror/MegEngine/imperative/python/tools/gen_ops.py:297
class RequireInputDynamicStorage(PodOpVisitor):
    param_names = ("param",)
    name = "RequireInputDynamicStorage"

    def __init__(self, *, param=None, name=None, comp_node=None, config=None, **kwargs):
        config = config or Config()
        if name:
            config.name = name
        if comp_node:
            config.comp_node = comp_node
        self.config = config

        self.param = helper.make_param(param, param_defs.Empty, kwargs)
        assert not kwargs, "extra kwargs: {}".format(kwargs)


